/*
 * HelloWorldServlet.java - jbig2-servlet
 * (C) 2019 DIGIBS S.L.
 * Todos los derechos reservados.
 */
package com.digibis;

import java.awt.image.BufferedImage;
import java.io.IOException;
import java.io.InputStream;
import java.io.PrintWriter;
import java.util.Arrays;
import java.util.Iterator;

import javax.imageio.ImageIO;
import javax.imageio.ImageReadParam;
import javax.imageio.ImageReader;
import javax.imageio.stream.ImageInputStream;
import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.lang3.tuple.ImmutablePair;
import org.apache.commons.lang3.tuple.Pair;

/**
 *
 */
public class HelloWorldServlet
    extends HttpServlet
{
    private static final long serialVersionUID = -6590308087774003501L;

    private String message = "";

    @Override
    public void init()
        throws ServletException
    {
        ImageIO.scanForPlugins();
        message += Arrays.toString(ImageIO.getReaderFormatNames());
        message += "<br>";
        this.readFile();
    }

    @Override
    protected void doGet(HttpServletRequest httpServletRequest, HttpServletResponse httpServletResponse)
        throws ServletException, IOException
    {
        message = "";
        message += Arrays.toString(ImageIO.getReaderFormatNames());
        message += "<br>";
        this.readFile();

        // Set response content type
        httpServletResponse.setContentType("text/html");

        // Actual logic goes here.
        PrintWriter out = httpServletResponse.getWriter();
        out.println("<h1>" + message + "</h1>");
    }

    @Override
    public void destroy()
    {
        // do nothing.
    }

    private void readFile()
    {
        ServletContext context = this.getServletContext();
        try (InputStream inputStream = context.getResourceAsStream("/WEB-INF/classes/20123110001.jb2")) {
            message += "Size: " + inputStream.available() + "bytes <br>";
            System.out.print(message);

            Pair<BufferedImage, String> image = this.readImageFromStream(inputStream);
            this.message += String.format("Format: %s <br> W: %s H: %s", image.getRight(), image.getLeft().getWidth(),
                image.getLeft().getHeight());
        } catch (IOException ex) {
            System.out.printf("Error reading JBIG2 file : %s", ex);
        }

    }

    /**
     * Lee una imgen desde un stream de entrada
     *
     * @param inputStream Stream con la imgen a leer
     * @return Par con la imgen y el formato real de la imgen
     * @throws IOException
     */
    private Pair<BufferedImage, String> readImageFromStream(InputStream inputStream)
        throws IOException
    {
        try (ImageInputStream imageInputStream = ImageIO.createImageInputStream(inputStream)) {
            ImageReader reader = this.getReader(imageInputStream);
            return this.readImage(imageInputStream, reader);
        }
    }

    private ImageReader getReader(ImageInputStream imageInputStream)
    {
        Iterator<ImageReader> readersIterator = ImageIO.getImageReaders(imageInputStream); // .getImageReadersByFormatName("jbig2");
        if (!readersIterator.hasNext()) {
            throw new IllegalArgumentException("Reader de ImageIO invalido o no encontrado");
        }
        return readersIterator.next();
    }

    private Pair<BufferedImage, String> readImage(ImageInputStream imageInputStream, ImageReader reader)
        throws IOException
    {
        try {
            reader.setInput(imageInputStream);
            // Optionally, listen for read warnings, progress, etc.
            // reader.addIIOReadWarningListener(...);
            // reader.addIIOReadProgressListener(...);

            ImageReadParam param = reader.getDefaultReadParam();
            // Finally read the image, using settings from param
            BufferedImage image = reader.read(0, param);
            return new ImmutablePair<>(image, reader.getFormatName());
        } finally {
            reader.dispose();
        }
    }
}
